import speech_recognition as sr
import pyttsx3
import datetime
import pywhatkit
import wikipedia
import pyjokes
import random
from gnewsclient import gnewsclient

__custom_joke = []
__engine = pyttsx3.init()
__voices = __engine.getProperty("voices")
__engine.setProperty("voice", __voices[1].id) # 40 is for en_IN, Female voice

def stop():
  __engine.stop()

def get_voice_id(voice_num):
  global __engine, __voices
  num_to_name = {
    0: 'Alex',    
    7: 'Daniel',  
    10: 'Fiona',   
    11: 'Fred',    
    17: 'Karen',   
    28: 'Moira',   
    32: 'Rishi',   
    33: 'Samantha',
    37: 'Tessa',   
    40: 'Veena',   
    41: 'Victoria'
  }

  i = 0
  for voice in __voices:
    if voice_num in num_to_name and voice.name == num_to_name[voice_num]:            
      return i
    elif voice_num not in num_to_name and 'en_IN' in voice.languages:
      return i
    i += 1

  return 1 # default value.

def set_voice(voice_num):
  global __engine, __voices

  v_id = 1
  try:
    v_id = get_voice_id(voice_num)
  except:
    pass
  __engine.setProperty("voice", __voices[v_id].id)
    

def add_custom_joke(joke):
  global __custom_joke
  if joke == None or joke.strip() == '' or joke in __custom_joke:
    return
  __custom_joke.append(joke)

############# Misc Functions #################
def erase_from_voice(voice, *phrases, debug = False):
  if voice == None or voice.strip() == '': 
    return ""

  if phrases == None: 
    return ""

  voice_filtered = voice.strip()

  for arg in phrases:
    voice_filtered = voice_filtered.replace(arg,"").strip()
  
  if debug:
    print("Voice:", voice)
    print("Phrase:", phrase)
    print("Voice filtered:", voice_filtered)
  
  return voice_filtered

def get_joke():
  return pyjokes.get_joke()

def get_custom_joke():
  global __custom_joke
  return random.choice(__custom_joke)

############# Speaking #################
def speak(phrase, *args, debug = True):
  if phrase == None:
    return
  phrase = str(phrase).strip()
  if phrase == '':
    return
  
  if "\n" in phrase:
    [speak_single(curr, debug) for curr in phrase.split("\n")]
  else:
    speak_single(phrase)

  for curr in args:
    if curr == None:
      return
    curr = str(curr).strip()
    if curr == '':
      return
    if "\n" in curr:
      [speak_single(curr_, debug) for curr_ in curr.split("\n")]
    else:
      speak_single(curr)


    
def speak_single(phrase, debug = True):
  if phrase == None or phrase.strip() == '':
    return
  phrase = phrase.strip()

  if debug:
    print(phrase)
  __engine.say(phrase)
  __engine.runAndWait()
############# Listening #################
def listen():
  r = sr.Recognizer()
  voice_input = ""
  try:
    print("Listening...")
    r.pause_threshold = 1
    with sr.Microphone() as source:
      voice = r.listen(source)
      voice_input = r.recognize_google(voice, language ='en-in')
      voice_input = voice_input.lower()
  except Exception as e:
    print("Error in taking voice input", e)
  return voice_input

############# Google News #################
def get_google_news(debug = False):
  client = gnewsclient.NewsClient(language = 'english',
                  location = 'India',
                  max_results=3)
  news_to_str = ''
  for news in client.get_news():
    news_to_str += (news['title'] + ".\n")

  if debug:
    print("News To Str: ", news_to_str)
  return news_to_str

############# Time Functions #################
def get_time_in_gmt(debug = False):
  time = datetime.datetime.now(datetime.timezone.utc)
  if debug:
    print(time)
  return time

def get_delta_time(hours, minutes):
  return datetime.timedelta(0, (hours * 60 + minutes) * 60)

def convert_time(utc_time, code, debug = False):
  hours = 5
  minutes = 30

  if "ist" in code:
    pass
  elif "aedt" in code:
    hours = 11
    minutes = 0
  
  elif "pdt" in code:
    hours = -7
    minutes = 0

  elif "gmt" in code or "utc" in code:
    hours = 0
    minutes = 0

  elif "cst" in code:
    hours = -6
    minutes = 0

  elif "brt" in code or "brst" in code:
    hours = -3
    minutes = 0

  elif "sast" in code:
    hours = 2
    minutes = 0 

  elif "jst" in code:
    hours = 9
    minutes = 0
  else:
    print("hours: ", hours)
    print("minutes: ", minutes)
    print("code: ", code)
    print("utc_time: ", utc_time)
    raise Exception("Conversion not found to the required code")

  if debug == True:
    print("hours: ", hours)
    print("minutes: ", minutes)
    print("code: ", code)
    print("utc_time: ", utc_time)

  time_new = (utc_time + get_delta_time(hours, minutes))
  return time_new.strftime("%I:%M %p")

############# Playing Song #################
def play_on_youtube(song):
  speak("playing " + song)
  pywhatkit.playonyt(song)



import setuptools
import os
import platform

version = '1.1.9'
setuptools.setup(
  name = 'kidbit_assistant',
  version = version,
  license='MIT',
  description = 'Design your Own Assistant',
  author = 'KidBit',
  author_email = 'kidbitacademy@gmail.com',   
  url = 'https://github.com/KidBit-Academy/kidbit_assistant',
  download_url = 'https://github.com/KidBit-Academy/kidbit_assistant/archive/refs/tags/v' + version + '.tar.gz',
  keywords = ['ALEXA', 'BOT', 'CODING', 'KIDS', 'KIDBIT', 'ASSISTANT'],  
  install_requires=[        
          'SpeechRecognition==3.8.1',
          'pywhatkit==5.3',
          'wikipedia==1.4.0',
          'pyjokes==0.6.0',
          'gnewsclient==1.12',
          'pyttsx3==2.90',
          'Levenshtein==0.18.1',
          'Flask==2.0.3',
          'pipwin==0.5.2; platform_system=="Windows"',
          'pyaudio==0.2.11; platform_system!="Windows"'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
  packages=["kidbit_assistant"],
  python_requires=">=3.6",
)

if platform.system() == 'Windows':
  os.system("pipwin install pyaudio")


from main import *

def get_time(voice):
    time_in_ist = "" 
    try:
        code = "ist"
        time_in_gmt = get_time_in_gmt()
        time_in_ist = convert_time(time_in_gmt, code)
    except:
        print("Cannot get time")
    
    return time_in_ist

add_custom_joke("Joke 1")
add_custom_joke("Joke 2")

while True:
    voice = listen()
    set_voice(40)
    print(voice)
    try:
        if "alexa" in voice:
            if "news" in voice:
                speak("Fetching Live News")
                news = get_google_news()
                speak(news)

            elif "time" in voice:
                time = get_time(voice)
                speak(time)

            elif "joke" in voice:
                joke = get_custom_joke()
                speak(joke)

            elif "play song" in voice:
                speak("Playing song")
                song = erase_from_voice(voice, "alexa", "play song")
                play_on_youtube(song)
                print("Enjoy")

            elif "where are we" in voice:
                speak("We are in India")

            elif "quit" in voice or "exit" in voice:
                speak("Goodbye! Have a great day!")
                stop()
                break
            
            elif "calculate" in voice:
                speak("What do you want to do?")
                voice2 = listen()
                if "add" in voice2:
                    speak("Speak number 1")
                    n1 = int(listen())
                    speak("Number 1 is ", n1)
                    speak("Speak number 2")
                    n2 = int(listen())
                    speak("Number 2 is ", n1)
                    speak("Addition of two numbers is ", n1 + n2)

                elif "subtract" in voice2:
                    speak("Speak number 1")
                    n1 = int(listen())
                    print("Number 1 is:", n1)
                    speak("Speak number 2")
                    n2 = int(listen())
                    print("Number 2 is:", n2)
                    speak("Subtraction of two numbers is ", n1 - n2)

            else:
                speak("I don't understand. Please try again.")
        else:
            speak("Did you wish to ask something? Try saying something with alexa in it.")
    except:
        speak("There was a fatal error. GoodBye")


